#!/bin/sh
#
#     analyse_pk3s.sh - Analyze .PK3 files for errors and conflicts
#
#     Copyright(C) 2004 - Jack Perdue (a.k.a. Silicon Slick)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

INSTALLDIR="/home/ailmanki/butler/backend/bin/analyse_pk3s/etmain"	# where the game is installed (clean install)
SDPK3DIR="/home/ailmanki/butler/backend/bin/analyse_pk3s/sdpk3"   # where developer PK3s are
DEVELOPERPK3S="astro-skies.pk3 common.pk3 lights.pk3 mapmedia.pk3 radiant.pk3" # GtkRadiant pk3s - last is a manual zip of etmain/models+scripts+maps
OTHERIPPK3DIR="/home/ailmanki/butler/backend/bin/analyse_pk3s/other"

BYPASSTEXTURES=0	# debug - bypass texture checks
BYPASSSOUNDS=0		# debug - bypass sound checks
BYPASSPAK0CHECK=0
BYPASSDEVELOPERCHECK=0
BYPASSMAPCONFLICTCHECK=0
BYPASSENTITYCHECK=0
BYPASSOTHERIPCHECK=0
PK3PREFIX=""		# debug - limit pk3s searched

TMPDIR="/tmp/ap-$$.tmp" 
TMPDUMP="$TMPDIR/dumpvalues"

STATICDIR="/tmp/analyzepk3s"
PAK0LISTRAW="$STATICDIR/pak0listraw.tmp"	# listing of pak0,pk3 (this script is limited to pak0.pk3 for now... needs work for RTCW)
PAK0LIST="$STATICDIR/pak0list.tmp"		# listing of pak0.pk3 in lowercase (this script is limited to pak0.pk3 for now... needs work for RTCW)
PAK0FILES="$STATICDIR/pak0files"		# unzipped copy of pak0.pk3 files (for comparisons)
PAK0SHADERLIST="$STATICDIR/pak0shaderlist.tmp"	# shaders referenced in pak0.pk3
PAK0MODELSTRINGS="$STATICDIR/pak0modelstrings.tmp"	# ?

SDPK3LISTDIR="$STATICDIR/sdpk3listings"	# listing of radiant files
SDPK3FILES="$STATICDIR/sdpk3files"		# unzipped copy of radiant files (for comparisons)

OTHERIPPK3LISTDIR="$STATICDIR/otherIPpk3listings"
OTHERIPPK3FILES="$STATICDIR/otherIPpk3files"

LISTDIR="$TMPDIR/pk3listings"		# listing of all pk3s being examined in lowercase

PK3LISTRAW="$TMPDIR/pk3listraw.tmp"	# listing of pk3 being examined (mixed case if the author made that mistake)
PK3LIST="$TMPDIR/pk3list.tmp"		# listing of pk3 being examined in lowercase
PK3FILES="$TMPDIR/pk3files"		# unzipped of copy of pk3 being examined files
PK3SHADERLIST="$TMPDIR/pk3shaderlist.tmp"  # listing of shaders in pk3 being examined
BSPSTRINGS="$TMPDIR/pk3bspstrings.tmp"		# model/texture strings referenced by .bsp in .pk3 being examined
PK3MODELSTRINGS="$TMPDIR/pk3modelstrings.tmp"	# ?

#
#  a couple of routines to save/retain values due to scoping of "echo/cat/whatever | while read" construct
#
dumpvalues(){
  echo "$BLOAT $STATUS_MINORNOTE $STATUS_MINORWARNING $STATUS_LEGACYWARNING $STATUS_WARNING $STATUS_MAJORWARNING $STATUS_FATALERROR $STATUS_SHADERWARNING $STATUS_SOUNDWARNING $STATUS_CONFLICTWARNING $STATUS_IMAGEWARNING $STATUS_MODELWARNING $STATUS_IPWARNING" > $TMPDUMP
  #echo "Dumped($1): $BLOAT - $STATUS_MINORNOTE - $STATUS_MINORWARNING - $STATUS_LEGACYWARNING - $STATUS_WARNING - $STATUS_MAJORWARNING - $STATUS_FATALERROR" 
  #echo -n "O: "
  #cat $TMPDUMP
} # dumpvalues()

loadvalues(){
  #echo -n "I: "
  #cat $TMPDUMP
  read BLOAT STATUS_MINORNOTE STATUS_MINORWARNING STATUS_LEGACYWARNING STATUS_WARNING STATUS_MAJORWARNING STATUS_FATALERROR STATUS_SHADERWARNING STATUS_SOUNDWARNING STATUS_CONFLICTWARNING STATUS_IMAGEWARNING STATUS_MODELWARNING STATUS_IPWARNING < $TMPDUMP
  #echo "Loaded($1): $BLOAT - $STATUS_MINORNOTE - $STATUS_MINORWARNING - $STATUS_LEGACYWARNING - $STATUS_WARNING - $STATUS_MAJORWARNING - $STATUS_FATALERROR" 
  #cat $TMPDUMP | read BLOAT STATUS_MINORNOTE STATUS_MINORWARNING STATUS_LEGACYWARNING XXX STATUS_MAJORWARNING STATUS_FATALERROR
  #echo "Loaded($1): $BLOAT - $STATUS_MINORNOTE - $STATUS_MINORWARNING - $STATUS_LEGACYWARNING - $XXX - $STATUS_MAJORWARNING - $STATUS_FATALERROR" 
} # loadvalues()

checkcampaign() {
  # see if a campaign was included (waste of bandwidth)
  cat $PK3LIST | grep "scripts/.*\.campaign" >& /dev/null
  if [ $? -eq 0 ]
  then
    echo "$PK3NAME - **** MAJOR WARNING: INCLUDES CAMPAIGN SCRIPTS (forcing download) ****"
    echo -n "$PK3NAME - "
    cat $PK3LIST | grep "scripts/.*\.campaign"
    let STATUS_MAJORWARNING=$STATUS_MAJORWARNING+1
  fi
} # checkcampaign()

checkbspname(){
  # verify .bsp (if there is one) matches .pk3 name
  if [ "$BSP" != "" ]
  then
    CHECKBSPNAME=`echo $BSP | sed 's,maps/,,g' | sed 's,bsp,pk3,g'`
    if [ "$CHECKBSPNAME" != "$PK3NAME" ]
    then
      #echo $PK3NAME $BSP $CHECKBSPNAME
      echo "$PK3NAME - **** MAJOR WARNING: BSP doesn't match PK3 name (very confusing) ****"
      echo "$PK3NAME - $BSP"
      let STATUS_MAJORWARNING=$STATUS_MAJORWARNING+1
    fi
  fi
} # checkbspname()

checkbspshaders(){
  # check for shaders referenced in BSP
  dumpvalues "aa"
  cat $BSPSTRINGS | grep "textures/" | sed "s,.*textures/,textures/,g" | sort | uniq | while read BSPSHADER
  do
    loadvalues "bb"
    grep $BSPSHADER $PK3SHADERLIST $PAK0SHADERLIST $PK3LIST $PAK0LIST >& /dev/null
    if [ $? -ne 0 ]
    then
      echo "$PK3NAME - *** Shader Warning: Could not locate shader $BSPSHADER referenced in $BSP ***"
      let STATUS_SHADERWARNING=$STATUS_SHADERWARNING+1
    fi
    dumpvalues "bb"
  done
  loadvalues "aa"
} # checkbspshaders()

checkbspmodels(){
  # check for models referenced in BSP
  dumpvalues "cc"
  cat $BSPSTRINGS | grep "models/" | sed 's,.*models/,models/,g' | sed 's,",,g' | sort | uniq | while read BSPMODEL
  do
    loadvalues "dd"
    grep $BSPMODEL $PAK0LIST $PK3LIST $PAK0MODELSTRINGS $PK3MODELSTRINGS >& /dev/null
    if [ $? -ne 0 ]
    then
      echo "$PK3NAME -  *** Model Warning: Could not locate model $BSPMODEL referenced in $BSP ***"
      let STATUS_MODELWARNING=$STATUS_MODELWARNING+1
    fi
    dumpvalues "dd"
  done
  loadvalues "cc"
} # checkbspmodels()

printreport(){
  let PERFECT=1
  echo -n "$PK3NAME -            SUMMARY: "
  if [ $STATUS_FATALERROR -ne 0 ]
  then
    echo -n "$STATUS_FATALERROR Fatal errors   "
    let PERFECT=0
  fi
  if [ $STATUS_MAJORWARNING -ne 0 ]
  then
    echo -n "$STATUS_MAJORWARNING Major warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_WARNING -ne 0 ]
  then
    echo -n "$STATUS_WARNING Warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_IPWARNING -ne 0 ]
  then
    echo -n "$STATUS_IPWARNING IP Warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_SHADERWARNING -ne 0 ]
  then
    echo -n "$STATUS_SHADERWARNING Shader warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_MODELWARNING -ne 0 ]
  then
    echo -n "$STATUS_MODELWARNING Model warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_IMAGEWARNING -ne 0 ]
  then
    echo -n "$STATUS_IMAGEWARNING Image warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_SOUNDWARNING -ne 0 ]
  then
    echo -n "$STATUS_SOUNDWARNING Sound warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_CONFLICTWARNING -ne 0 ]
  then
    echo -n "$STATUS_CONFLICTWARNING Conflict warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_LEGACYWARNING -ne 0 ]
  then
    echo -n "$STATUS_LEGACYWARNING Legacy warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_MINORWARNING -ne 0 ]
  then
    echo -n "$STATUS_MINORWARNING Minor warnings   "
    let PERFECT=0
  fi
  if [ $STATUS_MINORNOTE -ne 0 ]
  then
    echo -n "$STATUS_MINORNOTE Minor notes   "
    let PERFECT=0
  fi
  if [ $PERFECT -eq 1 ]
  then
    echo "Very nice job... no problems found"
  else
    echo ""
  fi
} # printreport()

remembermodels(){
  # if model, remember strings
  echo $PK3FILE | grep "models/.*\.md3" >& /dev/null
  if [ $? -eq 0 ]
  then
    echo "---- $PK3FILE ----" >> $PK3MODELSTRINGS
    unzip -p $PK3NAME $PK3RAWFILE | strings | grep models | sort | uniq >> $PK3MODELSTRINGS
  fi
} # remembermodels()

checkmapscriptentities(){
  if [ $BYPASSENTITYCHECK -eq 0 ]
  then
    # if map script, check for entities
    echo $PK3FILE | grep "maps/.*\.script" >& /dev/null
    if [ $? -eq 0 ]
    then
      BADSCRIPT=0
      for entity in `unzip -aap $PK3NAME $PK3RAWFILE | sed "s,//.*,,g" | grep alertentity | sed 's,.*alertentity ,,g' | sort | uniq`
      do
        cat $BSPSTRINGS | grep "\"$entity\"" >& /dev/null
        if [ $? -ne 0 ]
        then
          echo "$PK3NAME - **** MAJOR WARNING: Unable to locate $PK3FILE's entity $entity in $BSP ****"
          let STATUS_MAJORWARNING=$STATUS_MAJORWARNING+1
        fi
      done # for each entity
    fi # end if map script
  fi # end if bypass entity check
} # checkmapscriptentities()

checksounds(){
  if [ $BYPASSSOUNDS -eq 0 ]
  then
      # if a sound file, check wavs
      echo $PK3FILE | grep -e "sound/scripts/.*\.sounds" -e "sound/maps/.*\.sps" >& /dev/null
      if [ $? -eq 0 ]
      then
        BADSOUND=0
        dumpvalues "I"
        for rawsound in `unzip -aap $PK3NAME $PK3RAWFILE |  sed "s,//.*,,g" | grep -e "sound.*wav" -e "sound.*WAV" | sed 's,",,g' | sed 's,.*noise.*sound,sound,g' | sed 's,.*sound.*sound,sound,g' |  sort | uniq`
        do
          loadvalues "J"
                                                                                                                                                             
          sound=`echo $rawsound | tr [:upper:] [:lower:]`
          if [ "$sound" != "$rawsound" ]
          then
            echo "$PK3NAME -  ** Legacy warning: the name $rawsound contains uppercase letters which should be avoided **"
            let STATUS_LEGACYWARNING=$STATUS_LEGACYWARNING+1
          fi
                                                                                                                                                             
          cat $PK3LIST | grep "$sound" >& /dev/null
          if [ $? -ne 0 ]
          then
            cat $PAK0LIST | grep "$sound" >& /dev/null
            if [ $? -ne 0 ]
            then
              echo "$PK3NAME -  *** Sound Warning: UNABLE TO LOCATE $sound in $PK3FILE ****"
              BADSOUND=1
              let STATUS_SOUNDWARNING=$STATUS_SOUNDWARNING+1
            fi
          fi
          dumpvalues "J"
        done
        loadvalues "I"
        if [ $BADSOUND -ne 0 ]
        then
          echo "$PK3NAME - **** MAJOR WARNING: SOUND $PK3FILE REFERENCES NON-EXISTENT FILES (see above) ****"
          let STATUS_MAJORWARNING=$STATUS_MAJORWARNING+1
        fi
      fi # if sound file
  fi # if bypass sound check
} # checksounds()

checktextures(){
  # if a shader file, check textures and levelshots
  echo "$PK3FILE" | grep "scripts/.*\.shader" >& /dev/null
  if [ $? -eq 0 ]
  then
    unzip -aap $PK3NAME $PK3RAWFILE | sed 's,//.*,,g' | grep "^ *textures" | grep -v -e jpg -e tga | sort | uniq  >> $PK3SHADERLIST
    if [ $BYPASSTEXTURES -eq 0 ]
    then
  
        BADSHADER=0
        #echo "Shader script $PK3FILE"
        # check for missing textures
        dumpvalues "G"
        for rawtexture in `unzip -aap $PK3NAME $PK3RAWFILE |  sed "s,//.*,,g" | grep -e "textures.*tga"  -e "textures.*TGA"  -e "textures.*jpg"  -e "textures.*JPG" -e "levelshots.*tga" -e "levelshots.*TGA" -e "levelshots.*jpg" -e "levelshots.*JPG" | sed 's,.*textures,textures,g' | sed 's,.*levelshots,levelshots,g' | sort | uniq`
        do
          loadvalues "H"
                                                                                                                                                             
          texture=`echo $rawtexture | tr [:upper:] [:lower:]`
                                                                                                                                                             
          if [ "$rawtexture" != "$texture" ]
          then
            echo "$PK3NAME -  ** Legacy warning: the name $PK3RAWFILE contains uppercase letters which should be avoided **"
            let STATUS_LEGACYWARNING=$STATUS_LEGACYWARNING+1
          fi
                                                                                                                                                             
          echo $texture | grep "\.jpg" >& /dev/null
          if [ $? -eq 0 ]
          then
            echo $texture | grep "textures" >& /dev/null
            if [ $? -eq 0 ]
            then
              echo "$PK3NAME -  ** Legacy warning: texture $texture in $PK3FILE should use .tga extension (even if .jpg file) **"
              let STATUS_LEGACYWARNING=$STATUS_LEGACYWARNING+1
            fi
          fi
                                                                                                                                                             
                                                                                                                                                             
          #echo "Check for texture $texture"
          MISSING=1
          for textension in jpg JPG tga TGA
          do
            textpre=`echo $texture | sed 's,\.tga,,g' | sed 's,\.TGA,,g' | sed 's,\.jpg,,g' | sed 's,\.JPG,,g'`
            trytexture="$textpre.$textension"
                                                                                                                                                             
            #echo Try $texture $trytexture
            cat $PK3LIST | grep "$trytexture" >& /dev/null
            if [ $? -eq 0 ]
            then
              FOUNDINPK3="$PK3NAME"
              MISSING=0
              break
            fi
                                                                                                                                                             
            cat $PAK0LIST | grep "$trytexture" >& /dev/null
            if [ $? -eq 0 ]
            then
              FOUNDINPK3="pak0.pk3"
              MISSING=0
              break
            fi
                                                                                                                                                             
            # not everyone has these so if it doesn't matter if it is in SD pk3s
            #cat $SDPK3LIST | grep "$trytexture" >& /dev/null
            #if [ $? -eq 0 ]
            #then
            #  FOUNDINPK3="Editor .pk3s"
            #  MISSING=0
            #  break
            #fi
          done # for each extension
                                                                                                                                                             
          if [ $MISSING -eq 1 ]
          then
            echo "$PK3NAME -   ** Image Warning: UNABLE TO LOCATE $texture IN $PK3FILE **"
            BADSHADER=1
            let STATUS_IMAGEWARNING=$STATUS_IMAGEWARNING+1
          else
            if [ "$texture" != "$trytexture" ]
            then
              echo "$PK3NAME -         Minor note: $texture is $trytexture in $FOUNDINPK3"
              let STATUS_MINORNOTE=$STATUS_MINORNOTE+1
            fi
          fi
          dumpvalues "H"
        done # for each texture
        loadvalues "G"
                                                                                                                                                             
        if [ $BADSHADER -eq 1 ]
        then
          echo "$PK3NAME - **** MAJOR Warning: SHADER $PK3FILE REFERENCES NON-EXISTENT IMAGES (see above) ****"
          let STATUS_MAJORWARNING=$STATUS_MAJORWARNING+1
        fi

    fi # end if bypass
  fi # end if shader
} # checktextures()

checkmapconflicts(){
  if [ $BYPASSMAPCONFLICTCHECK -eq 0 ]
  then
    # check with conflicts with other maps
    # echo Check conflict $PK3FILE
    dumpvalues "E"
    grep "$PK3FILE" $LISTDIR/* | grep -v $PK3NAME | while read conpk3list condate consize confile
    do
      loadvalues "F"
      conpk3=`echo $conpk3list | sed "s,$LISTDIR/,,g" | sed 's,\.list,,g'`
      echo "$PK3NAME -   Conflict warning: $PK3FILE is also in $conpk3 - POTENTIAL CONFLICT"
      let STATUS_CONFLICTWARNING=$STATUS_CONFLICTWARNING+1
      dumpvalues "F"
    done
    loadvalues "E"
  fi
} # checkmapconflicts()

checkdeveloperfile(){
  if [ $BYPASSDEVELOPERCHECK -eq 0 ]
  then
    #echo -n "$PK3FILE is a Radiant developer file... check it in "
    SDPK3=`grep "$PK3FILE" $SDPK3LISTDIR/* | cut -f 1 -d ' ' | sed "s,$SDPK3LISTDIR/,,g" | sed 's,\.list.*,,g'`
    #echo SDPK3="$SDPK3"

    if [ ! -f $SDPK3FILES/$PK3FILE ]
    then
      unzip $SDPK3DIR/$SDPK3 $PK3FILE -d $SDPK3FILES > /dev/null
    fi
    unzip $PK3NAME $PK3FILE -d $PK3FILES > /dev/null
    file $PK3FILES/$PK3FILE | sed "s,$PK3FILES/$PK3FILE,,g" | grep "text" >& /dev/null
    if [ $? -eq 0 ]
    then
      mv -f $PK3FILES/$PK3FILE $PK3FILES/$PK3FILE.tmp
      cat $PK3FILES/$PK3FILE.tmp | dos2unix > $PK3FILES/$PK3FILE
    fi

    #echo "PK3FILE = $PK3FILES/$PK3FILE"
    #ls -al $PK3FILES/$PK3FILE
    #echo "SDPK3FILE = $SDPK3FILES/$PK3FILE"
    #ls -al $SDPK3FILES/$PK3FILE
    #echo diff  $SDPK3FILES/$PK3FILE $PK3FILES/$PK3FILE 
    diff  $SDPK3FILES/$PK3FILE $PK3FILES/$PK3FILE >& /dev/null
    if [ $? -ne 0 ]
    then
      echo "$PK3NAME - ****** FATAL ERROR: $PK3FILE is a deviation from the stock developer version - other maps that use it will either impact it or be impacted by it ******"
      diff $SDPK3FILES/$PK3FILE $PK3FILES/$PK3FILE
      let STATUS_FATALERROR=$STATUS_FATALERROR+1
    #else
    #  echo "$PK3NAME -    * Minor warning: $PK3FILE uses a standard name from Radiant and might get clobbered by poorly made maps *"
    #  let STATUS_MINORWARNING=$STATUS_MINORWARNING+1
    fi
  fi
} # checkdeveloperfile()

checkotherIP(){
  if [ $BYPASSOTHERIPCHECK -eq 0 ]
  then
    OTHERIPPK3=`grep "$PK3FILE" $OTHERIPPK3LISTDIR/* | head -1 | cut -f 1 -d ' ' | sed "s,$OTHERIPPK3LISTDIR/,,g" | sed 's,\.list.*,,g'`
    echo "$PK3NAME -    **** IP Warning: $PK3FILE appears to be stolen from $OTHERIPPK3"
    let STATUS_IPWARNING=$STATUS_IPWARNING+1
  fi
} # checkotherIP()

checkpak0file(){
  if [ $BYPASSPAK0CHECK -eq 0 ]
  then
        dumpvalues "C"
        cat $PAK0LIST | grep "$PK3FILE" | while read size0 date0 time0 file0
        do
          loadvalues "D"
          #echo pak0 $size0 $date0 $time0 $PK3FILE0
          if [ $size -eq $size0 -a "$time" = "$time0" -a "$date" = "$date0" ]
          then
            echo "$PK3NAME -    * Minor warning: $PK3FILE - same sizes and modification time - EXCESSIVE BLOAT *"
            let BLOAT=${BLOAT}+${size}
            let STATUS_MINORWARNING=$STATUS_MINORWARNING+1
          else
            if [ $size -ne $size0 ]
            then
              echo "$PK3NAME - ****** FATAL ERROR: FILE $PK3FILE IS IN CONFLICT WITH STOCK FILES (size: $size vs. $size0) ******"
              let STATUS_FATALERROR=$STATUS_FATALERROR+1
            else
              if [ ! -f $PAK0FILES/$PK3FILE ]
              then
                unzip $INSTALLDIR/pak0.pk3 $PK3FILE -d $PAK0FILES > /dev/null
              fi
              if [ ! -f $PK3FILES/$PK3FILE ]
              then
                unzip $PK3NAME $PK3FILE -d $PK3FILES > /dev/null
              fi
              diff $PAK0FILES/$PK3FILE $PK3FILES/$PK3FILE >& /dev/null
              if [ $? -eq 0 ]
              then
                if [ "$date" != "$date0" ]
                then
                  echo "$PK3NAME -    * Minor warning: $PK3FILE - different modification date ($date vs. $date0) but same file - EXCESSIVE BLOAT *"
                  let BLOAT=${BLOAT}+${size}
                  let STATUS_MINORWARNING=$STATUS_MINORWARNING+1
                else
                  if [ "$time" != "$time0" ]
                  then
                    echo "$PK3NAME -    * Minor warning: $PK3FILE - different modification time ($time vs. $time0) but same file - EXCESSIVE BLOAT *"
                    let BLOAT=${BLOAT}+${size}
                    let STATUS_MINORWARNING=$STATUS_MINORWARNING+1
                  fi
                fi
              else
                echo "$PK3NAME - ****** FATAL ERROR: FILE $PK3FILE IS IN CONFLICT WITH STOCK FILES ******"
                echo -n "$PK3NAME - "
                diff $PAK0FILES/$PK3FILE $PK3FILES/$PK3FILE
                let STATUS_FATALERROR=$STATUS_FATALERROR+1
              fi
            fi
          fi
          dumpvalues "D"
        done
        loadvalues "C"
  fi # if bypass
} # checkpak0file()

buildpak0lists(){
  # get base pak0.pk3 listing in mixed and lowercase

  if [ ! -f $PAK0LISTRAW -o ! -f $PAK0LIST ]
  then
    unzip -l $INSTALLDIR/pak0.pk3 | grep -v -e '----' -e 'Archive:' -e Length -e Created -e "files$" > $PAK0LISTRAW
    cat $PAK0LISTRAW | tr [:upper:] [:lower:] > $PAK0LIST
  fi
 
  if [ ! -f $PAK0SHADERLIST ]
  then 
    cat /dev/null > $PAK0SHADERLIST
    grep ".*\.shader" $PAK0LISTRAW | while read size date time file
    do
      unzip -aap $INSTALLDIR/pak0.pk3 $file  | sed 's,//.*,,g' | grep "^ *textures" | grep -v -e jpg -e tga | sort | uniq  >> $PAK0SHADERLIST
    done
  fi

  if [ ! -f $PAK0MODELSTRINGS ]
  then
    cat /dev/null > $PAK0MODELSTRINGS
    grep -e ".*\.md3" -e ".*\.md3" $PAK0LISTRAW | while read size date time file
    do
      #echo "prep $PK3FILE"
      unzip -p $INSTALLDIR/pak0.pk3 $file | strings | grep models | sort | uniq >> $PAK0MODELSTRINGS
    done
  fi
} # buildpak0lists()

builddeveloperlists(){
  # get list of SDradient editor files
  #for file in $DEVELOPERPK3S
  for file in `ls $SDPK3DIR/*.pk3`
  do
    base=`basename $file`
    if [ ! -f $SDPK3LISTDIR/$base.list ]
    then
      cat /dev/null > $SDPK3LISTDIR/$base.list
      unzip -l $file | grep -v -e '----' -e 'Archive:' -e Length -e Created -e "files$" | tr [:upper:] [:lower:] >> $SDPK3LISTDIR/$base.list
    fi
  done
} # builddeveloperlists

buildotherIPlists(){
  if [ ! -d $OTHERIPPK3DIR ]
  then
    mkdir -p $OTHERIPPK3DIR
  fi
  for file in `ls /usr/local/games/wolfenstein/main/*.pk3`
  do
    echo $file
    base=`basename $file`
    if [ ! -L $OTHERIPPK3DIR/wolfenstein-$base ]
    then
      ln -fs $file $OTHERIPPK3DIR/wolfenstein-$base
    fi
  done
 
  for file in `ls /usr/local/games/quake3/baseq3/pak*.pk3`
  do
    echo $file
    base=`basename $file`
    if [ ! -L $OTHERIPPK3DIR/quake3-$base ]
    then 
      ln -sf $file $OTHERIPPK3DIR/quake3-$base
    fi
  done

  for file in `ls $OTHERIPPK3DIR/*.pk3`
  do
    base=`basename $file`
    if [ ! -f $OTHERIPPK3LISTDIR/$base.list ]
    then
      cat /dev/null > $OTHERIPPK3LISTDIR/$base.list
      unzip -l $file | grep -v -e '----' -e 'Archive:' -e Length -e Created -e "files$" | tr [:upper:] [:lower:] >> $OTHERIPPK3LISTDIR/$base.list
    fi
  done
} # buildotherIPlists

buildpk3lists(){
  # get listing of all pk3s being analyzed in lower case
  touch $LISTDIR/junk.pk3.list  # need at least two files there so the grepping below works
  for file in `ls $PK3PREFIX*.pk3 | sort`
  do
    unzip -l $file | grep -v -e '----' -e 'Archive:' -e Length -e Created -e "files$" | tr [:upper:] [:lower:] > $LISTDIR/$file.list
  done
} # buildpk3lists()

checkforfile(){
  # $1 = file, $2 = description
  grep $1 $PK3LIST >& /dev/null
  if [ $? -ne 0 ]
  then
    echo "$PK3NAME -        *** Warning: Missing $1 - $2 ***"
    let STATUS_WARNING=$STATUS_WARNING+1
  fi 
} # check for file


#
# main() - Begin report
#

date				# timestamp report
echo ""
echo "Files to analyze:"
echo ""
#ls $PK3PREFIX*.pk3 | column -c 150 	# listing of files we'll examine
ls -l $PK3PREFIX*.pk3
echo ""

rm -rf /tmp/ap-[0-9]*.tmp
mkdir -p $TMPDIR	# make temp dirs
mkdir -p $STATICDIR
mkdir -p $LISTDIR
mkdir -p $SDPK3LISTDIR
mkdir -p $OTHERIPPK3LISTDIR
mkdir -p $SDPK3FILES
mkdir -p $OTHERIPPK3FILES
mkdir -p $PAK0FILES

buildpak0lists
builddeveloperlists
buildotherIPlists
buildpk3lists

# for each pk3 to analyze
for PK3NAME in `ls $PK3PREFIX*.pk3 | sort`
do
  # Initialize
  cat /dev/null > $PK3SHADERLIST
  cat /dev/null > $PK3MODELSTRINGS
  rm -rf $PK3FILES/*
  let BLOAT=0
  let STATUS_MINORNOTE=0
  let STATUS_MINORWARNING=0
  let STATUS_LEGACYWARNING=0
  let STATUS_SHADERWARNING=0
  let STATUS_CONFLICTWARNING=0
  let STATUS_SOUNDWARNING=0
  let STATUS_IMAGEWARNING=0
  let STATUS_MODELWARNING=0
  let STATUS_WARNING=0
  let STATUS_MAJORWARNING=0
  let STATUS_FATALERROR=0
  let STATUS_IPWARNING=0

  # Process
  echo ""
  echo "----- PK3 - $PK3NAME -----"
  echo ""
  unzip -l $PK3NAME | grep -v -e '----' -e 'Archive:' -e Length -e Created -e "files$" > $PK3LISTRAW
  cat $PK3LISTRAW | tr [:upper:] [:lower:] > $PK3LIST
  BSP=""
  let numbsps=`grep -c "maps.*bsp" $PK3LIST`
  if [ $numbsps -eq 0 ]
  then
    echo "$PK3NAME - No .bsp file found... not a map .pk3... bypassing checks"
    continue 
  else
    if [ $numbsps -gt 1 ]
    then
      echo "$PK3NAME - **** MAJOR WARNING: Includes more than one .BSP file??? ****"
      let STATUS_MAJORWARNING=$STATUS_MAJORWARNING+1
    else
       BSP=`cat $PK3LISTRAW | grep -e " maps.*bsp" -e " maps.*BSP" | sed 's,.*maps,maps,g'`
    fi
  fi
  cat /dev/null > $BSPSTRINGS
  cat /dev/null > $BSPSTRINGS.tmp
  dumpvalues "A0"
  for bsp in `cat $PK3LISTRAW | grep -e " maps.*bsp" -e " maps.*BSP" | sed 's,.*maps,maps,g'`
  do
    loadvalues "B0"
    unzip -p $PK3NAME $bsp | strings  > $BSPSTRINGS.tmp
    cat $BSPSTRINGS.tmp >> $BSPSTRINGS
    mapname=`echo $bsp | tr [:upper:] [:lower:] | sed 's,maps/,,g' | sed 's,\.bsp,,g'`
    #echo mapname is $mapname - bsp is $bsp
    checkforfile maps/${mapname}.script "Objective Mode Script"
    checkforfile maps/${mapname}_lms.script "LMS Mode Script"
    checkforfile maps/${mapname}.objdata "Map Objectives Listing"
    checkforfile maps/${mapname}_lms.objdata "LMS Mode Objectives Listing"
    checkforfile maps/${mapname}_tracemap.tga "Trace Map"
    checkforfile scripts/${mapname}.arena "Arena File"
    checkforfile scripts/${mapname}.shader "Shader File"
    for game in wolfmp wolfsw wolflms
    do

      #echo grep $game $BSPSTRINGS.tmp # >& /dev/null
      #grep $game $BSPSTRINGS.tmp # >& /dev/null
      #if [ $? -ne 0 ]
      #then
      #  echo "*******TEST********* bsp $bsp does not support $game"
      #fi
      unzip -aap $PK3NAME scripts/${mapname}.arena | grep $game >& /dev/null
      if [ $? -ne 0 ]
      then
        echo "*******TEST*********  scripts/${mapname}.arena does not support $game"
      fi
    done
    dumpvalues "B0"
  done
  loadvalues "A0"

  dumpvalues "A"
  cat $PK3LISTRAW | while read size date time PK3RAWFILE
  do
    loadvalues "B"
    PK3FILE=`echo "$PK3RAWFILE" | tr [:upper:] [:lower:]`
    if [ "$PK3FILE" != "$PK3RAWFILE" ]
    then
      echo "$PK3NAME -  ** Legacy warning: the name $PK3RAWFILE contains uppercase letters which should be avoided **"
      let STATUS_LEGACYWARNING=$STATUS_LEGACYWARNING+1
    fi
    echo "$PK3FILE" | grep " " >& /dev/null
    if [ $? -eq 0 ]
    then
      echo "$PK3NAME -         Minor note: the name \"$PK3RAWFILE\" contains spaces which is stupid and will break this script. :)"
      let STATUS_MINORNOTE=$STATUS_MINORNOTE+1
    fi

    echo "$PK3FILE" | grep readme >& /dev/null
    if [ $? -eq 0 ]
    then
      echo "$PK3NAME -         Minor note: ignoring README file \"$PK3RAWFILE\""
      let STATUS_MINORNOTE=$STATUS_MINORNOTE+1
      dumpvalues "B - continued"
      continue # next file
    fi

    echo "$PK3FILE" | grep thumbs.db >& /dev/null
    if [ $? -eq 0 ]
    then
      echo "$PK3NAME -    * Minor warning: nobody wants your thumbnail database \"$PK3RAWFILE\" -- EXCESSIVE BLOAT *"
      let STATUS_MINORWARNING=$STATUS_MINORWARNING+1
      let BLOAT=${BLOAT}+${size}
      dumpvalues "B - continued"
      continue # next file
    fi


    if [ $size -ne 0 ]
    then
      grep "$PK3FILE" $PAK0LIST >& /dev/null
      if [ $? -eq 0 ]
      then
        echo "$PK3NAME -        *** Warning: File $PK3FILE of size $size in $PK3NAME is in conflict with pak0.pk3 ***"
        let STATUS_WARNING=$STATUS_WARNING+1
        checkpak0file
      else # else, not a pak0.pk3
        grep "$PK3FILE" $SDPK3LISTDIR/* >& /dev/null
        if [ $? -eq 0 ]
        then
          checkdeveloperfile
        else
          grep "$PK3FILE" $OTHERIPPK3LISTDIR/* >& /dev/null
          if [ $? -eq 0 ]
          then
            checkotherIP
          else
            checkmapconflicts
          fi
        fi # found in SD pk3s
      fi # found in pak0.pk3

      checktextures 
      checksounds
      remembermodels
      checkmapscriptentities

    fi  # if size = 0
    dumpvalues "B"

  done # for each file in pk3
  loadvalues "A"

  checkbspshaders
  checkbspmodels

  if [ $BLOAT -ne 0 ]
  then
    let size=`du -bs $PK3NAME | cut -f 1`
    #echo BLOAT $BLOAT size $size 
    let percent=$BLOAT*105
    #echo percent $percent
    let percent=$percent/$size
    echo "$PK3NAME -        *** Warning: TOTAL EXCESSIVE BLOAT is $BLOAT bytes out of $size ($percent% BLOAT) ***"
    let STATUS_WARNING=$STATUS_WARNING+1
  fi

  checkcampaign
  checkbspname

  if [ $STATUS_IPWARNING -ne 0 ]
  then 
    echo "$PK3NAME - **** MAJOR WARNING: contains intellectual property protected by non-ET/Radiant copyrights"
    let STATUS_MAJORWARNING=$STATUS_MAJORWARNING+1
  fi
  printreport

done

# clean up
#rm -rf $TMPDIR
# exit without error
exit 0

# EOF


